from typing import List
import pyraylib
from pyraylib.colors import (
    GREEN,
    RAYWHITE,
    RED,
)
def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    MAX_BUILDINGS: int = 100
    window = pyraylib.Window(
        size=(SCREEN_WIDTH, SCREEN_HEIGHT), 
        title='pyraylib [core] example - 2d camera',
        fps=60,
    )
    player: pyraylib.Rectangle = pyraylib.Rectangle(400, 280, 40, 40)
    buildings: List[pyraylib.Rectangle] = []
    build_colors: List[pyraylib.Color] = []

    spacing = 0
    
    for i in range(MAX_BUILDINGS):
        rect = pyraylib.Rectangle()
        rect.width = pyraylib.get_random_value(50, 200)
        rect.height = pyraylib.get_random_value(100, 800)
        rect.x = -6000. + spacing
        rect.y = SCREEN_HEIGHT - 130.0 - rect.height
        spacing += rect.width
        buildings.append(rect)
        build_colors.append(pyraylib.get_random_color())
    
    camera = pyraylib.Camera2D(
        offset=(SCREEN_WIDTH/2., SCREEN_HEIGHT/2.),
        target=(player.x + 20., player.y + 20.),
        rotation=0.,
        zoom=1.,
    )

    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        # Player movement
        if pyraylib.is_key_down(pyraylib.Keyboard.RIGHT):
            player.x += 2
        elif pyraylib.is_key_down(pyraylib.Keyboard.LEFT):
            player -= 2

        # Camera target follows player
        camera.target = (player.x + 20, player.y + 20)

        # Camera rotation controls
        if pyraylib.is_key_down(pyraylib.Keyboard.A):
            camera.rotation -= 1
        elif pyraylib.is_key_down(pyraylib.Keyboard.S):
            camera.rotation += 1
        
        # Limit camera rotation to 80 degrees (-40 to 40)
        if camera.rotation > 40:
            camera.rotation = 40
        elif camera.rotation < -40:
            camera.rotation = -40
        
        # Camera zoom controls
        camera.zoom += pyraylib.get_mouse_wheel_move() * 0.05

        if camera.zoom > 3.:
            camera.zoom = 3.
        elif camera.zoom < 0.1:
            camera.zoom = 0.1
        
        # Camera reset (zoom and rotation)
        if pyraylib.is_key_pressed(pyraylib.Keyboard.R):
            camera.zoom = 1.
            camera.rotation = 0.
            
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        camera.begin_mode()
        for i in range(MAX_BUILDINGS):
            buildings[i].draw(build_colors[i])
        player.draw(RED)
        pyraylib.draw_line((camera.target.x, -SCREEN_HEIGHT*10) , (camera.target.x, SCREEN_HEIGHT*10), GREEN)
        pyraylib.draw_line((-SCREEN_WIDTH*10, camera.target.y), (SCREEN_WIDTH*10, camera.target.y), GREEN)
        camera.end_mode()
        pyraylib.draw_text("SCREEN AREA", 640, 10, 20, RED)
        window.end_drawing()

    # De-Initialization
    # TODO: Unload all loaded data (textures, fonts, audio) here!
    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()


import pyraylib
from pyraylib.colors import (
    BLUE,
    DARKGREEN,
    GREEN,
    RAYWHITE,
    LIGHTGRAY,
    GRAY,
    MAROON,
    DARKBLUE,
    PURPLE,
)
from enum import IntEnum
def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window(
        size=(SCREEN_WIDTH, SCREEN_HEIGHT), 
        title='pyraylib [core] example - basic screen manager',
        fps=60,
    )
    class GameScreen(IntEnum):
        """
        Window Status
        """
        LOGO = 0,
        TITLE = 1,
        GAMEPLAY = 2,
        ENDING = 3,

    current_screen = GameScreen.LOGO

    # TODO: Initialize all required variables and load all required data here!

    frames_counter = 0 # Useful to count frames
    rect = pyraylib.Rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)
    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        if current_screen == GameScreen.LOGO:
            # TODO: Update LOGO screen variables here!
            frames_counter += 1 # Count frames
            # Wait for 2 seconds (120 frames) before jumping to TITLE screen
            if frames_counter > 120:
                current_screen = GameScreen.TITLE
                del frames_counter
        
        elif current_screen == GameScreen.TITLE:
            # TODO: Update TITLE screen variables here!

            # Press enter to change to GAMEPLAY screen
            if pyraylib.is_key_pressed(pyraylib.Keyboard.ENTER) \
                or pyraylib.is_key_pressed(pyraylib.Keyboard.TAB):
                current_screen = GameScreen.GAMEPLAY

        elif current_screen == GameScreen.GAMEPLAY:
            # TODO: Update GAMEPLAY screen variables here!

            # Press enter to change to ENDING screen
            if pyraylib.is_key_pressed(pyraylib.Keyboard.ENTER) \
                or pyraylib.is_key_pressed(pyraylib.Keyboard.TAB):
                current_screen = GameScreen.ENDING

        elif current_screen == GameScreen.ENDING:
            # TODO: Update ENDING screen variables here!

            # Press enter to return to TITLE screen
            if pyraylib.is_key_pressed(pyraylib.Keyboard.ENTER) \
                or pyraylib.is_key_pressed(pyraylib.Keyboard.TAB):
                current_screen = GameScreen.TITLE
        
        else:
            pass
        
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        
        if current_screen == GameScreen.LOGO:
            # TODO: Draw LOGO screen here!
            pyraylib.draw_text("LOGO SCREEN", 20, 20, 40, LIGHTGRAY)
            pyraylib.draw_text("WAIT for 2 SECONDS...", 290, 220, 20, GRAY)

        elif current_screen == GameScreen.TITLE:
            # TODO: Draw TITLE screen here!
            rect.draw(GREEN)
            pyraylib.draw_text("TITLE SCREEN", 20, 20, 40, DARKGREEN)
            pyraylib.draw_text("PRESS ENTER or TAP to JUMP to GAMEPLAY SCREEN", 120, 220, 20, DARKGREEN)

        elif current_screen == GameScreen.GAMEPLAY:
            # TODO: Draw GAMEPLAY screen here!
            rect.draw(PURPLE)
            pyraylib.draw_text("GAMEPLAY SCREEN", 20, 20, 40, MAROON)
            pyraylib.draw_text("PRESS ENTER or TAP to JUMP to ENDING SCREEN", 130, 220, 20, MAROON)

        elif current_screen == GameScreen.ENDING:
            # TODO: Draw ENDING screen here!
            rect.draw(BLUE)
            pyraylib.draw_text("ENDING SCREEN", 20, 20, 40, DARKBLUE)
            pyraylib.draw_text("PRESS ENTER or TAP to RETURN to TITLE SCREEN", 120, 220, 20, DARKBLUE)

        window.end_drawing()

    # De-Initialization
    # TODO: Unload all loaded data (textures, fonts, audio) here!
    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()


# easings.py

#   Useful easing functions for values animation
#
#   How to use:
#   The four inputs t, b, c, d are defined as follows:
#   t = current time (in any unit measure, but same unit as duration)
#   b = starting value to interpolate
#   c = the total change in value of b that needs to occur
#   d = total time it should take to complete (duration)
#
#   Example:
#
#   current_time: int = 0;
#   duration: int = 100;
#   start_position_x: float = 0.0
#   final_position_x: float = 30.0
#   current_position_x: float = start_position_x
#
#   while currentPositionX < finalPositionX:
#       current_positionX = ease_sine_in(current_time, start_position_x, final_position_x - start_position_x, duration)
#       current_time += 1
#
#   A port of Robert Penner's easing equations to C (http://robertpenner.com/easing/)

#   Robert Penner License
#   ---------------------------------------------------------------------------------
#   Open source under the BSD License. 
#
#   Copyright (c) 2001 Robert Penner. All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without modification, 
#   are permitted provided that the following conditions are met:
#
#       - Redistributions of source code must retain the above copyright notice, 
#         this list of conditions and the following disclaimer.
#       - Redistributions in binary form must reproduce the above copyright notice, 
#         this list of conditions and the following disclaimer in the documentation 
#         and/or other materials provided with the distribution.
#       - Neither the name of the author nor the names of contributors may be used 
#         to endorse or promote products derived from this software without specific 
#         prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
#   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
#   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
#   IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
#   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
#   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
#   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
#   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
#   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
#   OF THE POSSIBILITY OF SUCH DAMAGE.
#   ---------------------------------------------------------------------------------

from math import asin, cos, pi, sin, sqrt

__all__ = [
    'ease_linear_none',
    'ease_linear_in',
    'ease_linear_out',
    'ease_linear_in_out',
    'ease_sine_in',
    'ease_sine_out',
    'ease_sine_in_out',
    'ease_circ_in',
    'ease_circ_out',
    'ease_circ_in_out',
    'ease_cubic_in',
    'ease_cubic_out',
    'ease_cubic_in_out',
    'ease_quad_in',
    'ease_quad_out',
    'ease_quad_in_out',
    'ease_expo_in',
    'ease_expo_out',
    'ease_expo_in_out',
    'ease_back_in',
    'ease_back_out',
    'ease_back_in_out',
    'ease_bounce_in',
    'ease_bounce_out',
    'ease_bounce_in_out',
    'ease_elastic_in',
    'ease_elastic_out',
    'ease_elastic_in_out',
]


# Linear Easing functions
def ease_linear_none(t: float, b: float, c: float, d: float) -> float:
    return c * t / d + b


def ease_linear_in(t: float, b: float, c: float, d: float) -> float:
    return c * t / d + b


def ease_linear_out(t: float, b: float, c: float, d: float) -> float:
    return c * t / d + b


def ease_linear_in_out(t: float, b: float, c: float, d: float) -> float:
    return c * t / d + b


# Sine Easing functions
def ease_sine_in(t: float, b: float, c: float, d: float) -> float:
    return -c * cos(t / d * (pi / 2)) + c + b


def ease_sine_out(t: float, b: float, c: float, d: float) -> float:
    return c * sin(t / d * (pi / 2)) + b


def ease_sine_in_out(t: float, b: float, c: float, d: float) -> float:
    return -c / 2 * (cos(pi * t / d) - 1) + b


# Circular Easing functions
def ease_circ_in(t: float, b: float, c: float, d: float) -> float:
    t /= d
    return -c * (sqrt(1 - t ** 2) - 1) + b


def ease_circ_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d - 1
    return c * sqrt(1 - t ** 2) + b


def ease_circ_in_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d * 2
    if t < 1:
        return -c / 2 * (sqrt(1 - t * t) - 1) + b
    else:
        t = t - 2
        return c / 2 * (sqrt(1 - t * t) + 1) + b


# Cubic Easing functions
def ease_cubic_in(t: float, b: float, c: float, d: float) -> float:
    t = t / d
    return c * (t ** 3) + b


def ease_cubic_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d - 1
    return c * ((t ** 3) + 1) + b


def ease_cubic_in_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d * 2
    if t < 1:
        return c / 2 * t * t * t + b
    else:
        t = t - 2
        return c / 2 * (t * t * t + 2) + b


# Quadratic Easing functions
def ease_quad_in(t: float, b: float, c: float, d: float) -> float:
    t = t / d
    return c * (t ** 2) + b


def ease_quad_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d
    return -c * t * (t - 2) + b


def ease_quad_in_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d * 2
    if t < 1:
        return c / 2 * (t ** 2) + b
    else:
        return -c / 2 * ((t - 1) * (t - 3) - 1) + b


# Exponential
def ease_expo_in(t: float, b: float, c: float, d: float) -> float:
    if t == 0.:
        return b
    else:
        return c * (2 ** (10 * (t / d - 1))) + b - c * 0.001


def ease_expo_out(t: float, b: float, c: float, d: float) -> float:
    if t == d:
        return b + c
    else:
        return c * 1.001 * (-(2 ** (-10) * t / d) + 1) + b


def ease_expo_in_out(t: float, b: float, c: float, d: float) -> float:
    if t == 0:
        return b
    if t == d:
        return b + c
    t = t / d * 2
    if t < 1:
        return c / 2 * (2 ** (10 * (t - 1))) + b - c * 0.0005
    else:
        t = t - 1
        return c / 2 * 1.0005 * (-(2 ** (-10 * t)) + 2) + b


# Back Easing functions
def ease_back_in(t: float, b: float, c: float, d: float) -> float:
    s: float = 1.70158
    t = t / d
    return c * t * t *((s + 1) * t - s) + b


def ease_back_out(t: float, b: float, c: float, d: float) -> float:
    s: float = 1.70158
    t = t / d - 1
    return c * (t * t  * ((s + 1) * t + s) + 1) + b


def ease_back_in_out(t: float, b: float, c: float, d: float) -> float:
    s: float = 1.70158
    s = s *1.525
    t = t / d * 2
    if t < 1:
        return c / 2 * (t * t * ((s + 1) * t - s)) + b
    else:
        t = t - 2
        return c / 2 * (t * t * ((s + 1) * t + s) + 2) + b


# Ease Bounce functions
def ease_bounce_in(t: float, b: float, c: float, d: float) -> float:
    return c - ease_bounce_out(d - t, 0, c, d) + b


def ease_bounce_out(t: float, b: float, c: float, d: float) -> float:
    t = t / d
    m = 7.5625
    n = 2.75
    if t < 1 / n:
        return c * (m * t * t) + b
    elif t < 2 / n:
        t = t - (1.5 / 2.75)
        return c * (m * t * t + 0.75) + b
    elif t < 2.5 / n:
        t = t - (2.25 / n)
        return c * (m * t * t + 0.9375) + b
    else:
        t = t - (2.625 / n)
        return c * (m * t * t + 0.984375) + b


def ease_bounce_in_out(t: float, b: float, c: float, d: float) -> float:
    if t < d / 2:
        return ease_bounce_in(t * 2, 0, c, d) * 0.5 + b
    else:
        return ease_bounce_out(t * 2 - d, 0, c, d) * 0.5 + c * 0.5 + b


# Ease Elastic functions
def ease_elastic_in(t: float, b: float, c: float, d: float) -> float:
    p: float = d * 0.3
    a: float = c
    s: float = p / (2 * pi) * asin(c / a)
    if t == 0.:
        return b
    t = t / d
    if t == 1:
        return b + c
    if a < abs(c):
        a = c
        s = p / 4

    t = t - 1
    return -(a * (2 ** (10 * t)) * sin((t * d - s) * (2 * pi) / p)) * b


def ease_elastic_out(t: float, b: float, c: float, d: float) -> float:
    p: float = d * 0.3
    a: float = c
    s: float = p / (2 * pi) * asin(c / a)
    if t == 0.:
        return b
    t = t / d
    if t == 1:
        return b + c
    if a < abs(c):
        a = c
        s = p / 4

    return a * (2 ** (-10 * t)) * sin((t * d - s) * (2 * pi ) / p) + c + b


def ease_elastic_in_out(t: float, b: float, c: float, d: float) -> float:
    if t == 0:
        return b
    t = t / d * 2
    if t == 2:
        return b + c

    p: float = d * (0.3 * 1.5)
    a: float = 0.0
    s: float = p / (2 * pi) * asin(c / a)

    if a < abs(c):
        a = c
        s = p / 4

    if t < 1:
        t = t - 1
        return -0.5 * (a * (2 ** (10 * t)) * sin((t * d - s) * (2 * pi) / p)) + b
    else:
        t = t - 1
        return a * (2 ** (-10 * t)) * sin((t * d - s) * (2 * pi) / p) * 0.5 + c + b



import pyraylib
from pyraylib.colors import (
    LIGHTGRAY,
    RAYWHITE
)

def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window((SCREEN_WIDTH, SCREEN_HEIGHT), 'pyraylib [core] example - basic window')
    # Set our game to run at 60 frames-per-second
    window.set_fps(60)

    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        # TODO: Update your variables here
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        pyraylib.draw_text('Congrats! You created your first window!', 190, 200, 20, LIGHTGRAY)
        window.end_drawing()

    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()


import pyraylib
from pyraylib.colors import (
    DARKGRAY,
    RAYWHITE,
    LIGHTGRAY,
    GRAY,
)

def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window(
        size=(SCREEN_WIDTH, SCREEN_HEIGHT), 
        title='pyraylib [core] example - drop files',
        fps=60
    )
    dropped_files = []
    color = pyraylib.Color(LIGHTGRAY)
    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        if pyraylib.is_file_dropped():
            dropped_files = pyraylib.get_dropped_files()
        
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)

        if dropped_files:
            pyraylib.draw_text("Dropped files:", 100, 40, 20, DARKGRAY)
            for idx , file in enumerate(dropped_files):
                if idx & 1 == 0:
                    rect = pyraylib.Rectangle(0, 85 + 40* idx, SCREEN_WIDTH, 40).draw(color.fade(0.5))
                else:
                    rect = pyraylib.Rectangle(0, 85 + 40* idx, SCREEN_WIDTH, 40).draw(color.fade(0.3))
                pyraylib.draw_text(file, 120, 100 + 40 * idx, 10, GRAY)
            pyraylib.draw_text("Drop new files...", 100, 110 + 40 * len(dropped_files), 20, DARKGRAY)
        else:
            pyraylib.draw_text("Drop your files to this window!", 100, 40, 20, DARKGRAY)
        window.end_drawing()

    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()


import pyraylib
from pyraylib.colors import (
    RAYWHITE,
    MAROON,
    LIGHTGRAY,
)
def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window(
        size=(SCREEN_WIDTH, SCREEN_HEIGHT), 
        title='pyraylib [core] example - generate random values',
        fps=60,
    )
    # Variable used to count frames
    frames_counter = 0
    # Get a random integer number between -8 and 5 (both included)
    rand_value = pyraylib.get_random_value(-8, 5)

    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        frames_counter += 1
        # Every two seconds (120 frames) a new random value is generated
        if frames_counter > 120:
            rand_value = pyraylib.get_random_value(-8, 5)
            frames_counter = 0

        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        pyraylib.draw_text("Every 2 seconds a new random value is generated:", 130, 100, 20, MAROON)
        pyraylib.draw_text(f"{rand_value}", 360, 180, 80, LIGHTGRAY)
        window.end_drawing()

    # De-Initialization
    # TODO: Unload all loaded data (textures, fonts, audio) here!
    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()


import pyraylib
from pyraylib.colors import (
    DARKGRAY,
    DARK_BLUE,
    LIME,
    MAROON,
    RAYWHITE
)

def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window((SCREEN_WIDTH, SCREEN_HEIGHT), 'pyraylib [core] example - mouse input')
    # Set our game to run at 60 frames-per-second
    window.set_fps(60)
    circle = pyraylib.Circle(
        center=pyraylib.Vector2(-100., -100.),
        radius=40,
        color=DARK_BLUE
    )
    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        circle.center = pyraylib.get_mouse_position()

        if pyraylib.is_mouse_button_pressed(pyraylib.MouseButton.LEFT_BUTTON):
            circle.color = MAROON
        elif pyraylib.is_mouse_button_pressed(pyraylib.MouseButton.MIDDLE_BUTTON):
            circle.color = LIME
        elif pyraylib.is_mouse_button_pressed(pyraylib.MouseButton.RIGHT_BUTTON):
            circle.color = DARK_BLUE
        
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        pyraylib.draw_text("move ball with mouse and click mouse button to change color", 10, 10, 20, DARKGRAY)
        circle.draw()
        window.end_drawing()

    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()


import pyraylib
from pyraylib.colors import (
    GRAY,
    LIGHTGRAY,
    MAROON,
    RAYWHITE
)

def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window((SCREEN_WIDTH, SCREEN_HEIGHT), 'pyraylib [core] example - mouse wheel')
    # Set our game to run at 60 frames-per-second
    window.set_fps(60)
    box = pyraylib.Rectangle(
        SCREEN_WIDTH / 2 - 40,
        SCREEN_HEIGHT/2 - 40,
        80,
        80,
    )
    scroll_speed = 2
    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        box.y = pyraylib.get_mouse_wheel_move() * scroll_speed
        
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        pyraylib.draw_text("Use mouse wheel to move the cube up and down!", 10, 10, 20, GRAY)
        pyraylib.draw_text(f"Box position Y: {box.y}", 10, 40, 20, LIGHTGRAY)
        box.draw(color=MAROON)
        window.end_drawing()

    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()



import pyraylib
from pyraylib.colors import (
    BEIGE,
    BLACK,
    BLUE,
    BROWN,
    DARKGRAY,
    GREEN,
    RED,
    SKY_BLUE,
    WHITE,
)


# Scene drawing
def draw_scene(texture_grid, camera_player_1, camera_player_2):
    count = 5
    spacing = 4

    # Grid of cube trees on a plane to make a "world"
    pyraylib.draw_plane((0, 0, 0 ), (50, 50), BEIGE) # Simple world plane

    for x in range(-count*spacing, (count*spacing)+1, spacing):
        for z in range(-count*spacing, (count*spacing)+1, spacing):
            pyraylib.draw_cube_texture(texture_grid, (x, 1.5, z), 1, 1, 1, GREEN)
            pyraylib.draw_cube_texture(texture_grid, (x, 0.5, z), 0.25, 1, 0.25, BROWN)
    
    # Draw a cube at each player's position
    pyraylib.draw_cube(camera_player_1.position, 1, 1, 1, RED)
    pyraylib.draw_cube(camera_player_2.position, 1, 1, 1, BLUE)

def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window(
        size=(SCREEN_WIDTH, SCREEN_HEIGHT), 
        title='pyraylib [core] example - split screen',
        fps=60
    )
    # Generate a simple texture to use for trees
    img = pyraylib.Image.checked(256, 256, 32, 32, DARKGRAY, WHITE)
    texture_grid = img.get_texture().set_filter(
        pyraylib.TextureFilter.ANISOTROPIC_16X
    ).set_wrap(pyraylib.TextureWrap.CLAMP)
    img.unload()

    # Setup player 1 camera and screen
    camera_player_1 = pyraylib.Camera(
        fovy=45.,
        up=(0, 1., 0),
        target=(0, 1., 0),
        position=(0, 1., -3.)
    )
    screen_player_1 = pyraylib.RenderTexture.load(SCREEN_WIDTH // 2, SCREEN_HEIGHT)

    # Setup player two camera and screen
    camera_player_2 = pyraylib.Camera(
        fovy=45.,
        up=(0, 1., 0),
        target=(0, 3., 0),
        position=(-3., 3., 0)
    )
    screen_player_2 = pyraylib.RenderTexture.load(SCREEN_WIDTH // 2, SCREEN_HEIGHT)
    
    # Build a flipped rectangle the size of the split view to use for drawing later
    split_screen_rect = pyraylib.Rectangle(
        0., 
        0., 
        screen_player_1.texture.width, 
        -screen_player_1.texture.height
        )
    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        """
        If anyone moves this frame, how far will they move based on the time since the last frame
        this moves thigns at 10 world units per second, regardless of the actual FPS
        """
        offset_this_frame = 10. * window.get_frame_time()

        # Move Player1 forward and backwards (no turning)
        if pyraylib.is_key_down(pyraylib.Keyboard.W):
            camera_player_1.position.z += offset_this_frame
            camera_player_1.target.z += offset_this_frame
        elif pyraylib.is_key_down(pyraylib.Keyboard.S):
            camera_player_1.position.z -= offset_this_frame
            camera_player_1.target.z -= offset_this_frame
        
        # Move Player2 forward and backwards (no turning)
        if pyraylib.is_key_down(pyraylib.Keyboard.UP):
            camera_player_2.position.x += offset_this_frame
            camera_player_2.target.x += offset_this_frame
        elif pyraylib.is_key_down(pyraylib.Keyboard.DOWN):
            camera_player_2.position.x -= offset_this_frame
            camera_player_2.target.x -= offset_this_frame

        # Draw
        # Draw Player1 view to the render texture
        # I can also use with statement to open and close render texture and camera3d mode
        with screen_player_1 as sp1:
            window.clear_background(SKY_BLUE)
            with camera_player_1 as cp1:
                draw_scene(texture_grid, camera_player_1, camera_player_2)
            pyraylib.draw_text("PLAYER1 W/S to move", 0, 0, 20, RED)
        
        # Draw Player2 view to the render texture
        # without using with statement
        screen_player_2.begin_mode()
        window.clear_background(SKY_BLUE)
        camera_player_2.begin_mode()
        draw_scene(texture_grid, camera_player_1, camera_player_2)
        camera_player_2.end_mode()
        pyraylib.draw_text("PLAYER2 UP/DOWN to move", 0, 0, 20, BLUE)
        screen_player_2.end_mode()

        # Draw both views render textures to the screen side by side
        window.begin_drawing()
        window.clear_background(BLACK)
        screen_player_1.texture.draw_rec(split_screen_rect, ( 0, 0), WHITE)
        screen_player_2.texture.draw_rec(split_screen_rect, (SCREEN_WIDTH / 2., 0.), WHITE)
        window.end_drawing()

    # De-Initialization
    screen_player_1.unload() # Unload render texture
    screen_player_2.unload() # Unload render texture
    texture_grid.unload()    # Unload texture
    window.close()           # Close window and OpenGL context

if __name__ == '__main__':
    main()


import pyraylib
from pyraylib.colors import (
    DARKGRAY,
    MAROON,
    RAYWHITE
)

def main():
    # Initialization
    SCREEN_WIDTH, SCREEN_HEIGHT = 800, 450
    window = pyraylib.Window((SCREEN_WIDTH, SCREEN_HEIGHT), 'pyraylib [core] example - keyboard input')
    # Set our game to run at 60 frames-per-second
    window.set_fps(60)
    circle = pyraylib.Circle(
        center=(SCREEN_WIDTH / 2, SCREEN_HEIGHT / 2),
        radius=50,
        color=MAROON
    )
    speed = 2.0
    # Main game loop
    while window.is_open(): # Detect window close button or ESC key
        # Update
        if pyraylib.is_key_down(pyraylib.Keyboard.RIGHT):
            circle.move(speed, 0) # circle.x += speed
        if pyraylib.is_key_down(pyraylib.Keyboard.LEFT):
            circle.move(-speed, 0) # circle.x -= speed
        if pyraylib.is_key_down(pyraylib.Keyboard.UP):
            circle.move(0, -speed) # circle.y -= speed
        if pyraylib.is_key_down(pyraylib.Keyboard.DOWN):
            circle.move(0, speed) # circle.y += speed
        
        # Draw
        window.begin_drawing()
        window.clear_background(RAYWHITE)
        pyraylib.draw_text("move the ball with arrow keys", 10, 10, 20, DARKGRAY)
        circle.draw()
        window.end_drawing()

    # Close window and OpenGL context
    window.close()

if __name__ == '__main__':
    main()